#!/usr/bin/env python3

"""
    Copyright 2023 carlkidcrypto, All rights reserved.
"""


import unittest
import requests_mock
import sys

sys.path.append("../")

from purpleair_api.PurpleAirAPIError import PurpleAirAPIError
from purpleair_api.PurpleAirLocalAPI import PurpleAirLocalAPI


class PurpleAirLocalAPITest(unittest.TestCase):
    def setUp(self):
        self.pala = PurpleAirLocalAPI(["192.168.1.2"])

    def tearDown(self):
        self.pala = None

    def test_request_local_sensor_data_ip_provided(self):
        """
        Test that we can provided a valid ip address. And that the url is formatted as expected.
        """

        # Setup
        fake_url_request = "http://192.168.1.2/json"

        # Action and Expected Result
        with requests_mock.Mocker() as m:
            m.get(
                fake_url_request,
                text='{"test": 5}',
                status_code=200,
            )
            self.pala.request_local_sensor_data()

    def test_request_local_sensor_data_none_ip_provided(self):
        """
        Test that providing no IP address results in a `PurpleAirAPIError`
        """

        # Setup,  Action, and Expected Result
        with self.assertRaises(PurpleAirAPIError):
            self.pala = PurpleAirLocalAPI(None)

    def test_request_local_sensor_data_empty_list_ip_provided(self):
        """
        Test that providing no IP address results in a `PurpleAirAPIError`
        """

        # Setup,  Action, and Expected Result
        with self.assertRaises(PurpleAirAPIError):
            self.pala = PurpleAirLocalAPI(None)

        with self.assertRaises(PurpleAirAPIError):
            self.pala = PurpleAirLocalAPI(["", ""])

        with self.assertRaises(PurpleAirAPIError):
            self.pala = PurpleAirLocalAPI({})


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
    Copyright 2024 carlkidcrypto, All rights reserved.
    A python3 class designed to fetch data from Purple Air's new API.
    https://api.purpleair.com/#api-welcome
"""

from purpleair_api.PurpleAirAPIHelpers import debug_log, send_url_get_request
from purpleair_api.PurpleAirAPIError import PurpleAirAPIError
from purpleair_api.PurpleAirReadAPI import PurpleAirReadAPI
from purpleair_api.PurpleAirWriteAPI import PurpleAirWriteAPI
from purpleair_api.PurpleAirLocalAPI import PurpleAirLocalAPI


class PurpleAirAPI(PurpleAirReadAPI, PurpleAirWriteAPI, PurpleAirLocalAPI):
    """
    The PurpleAirAPI class designed to send valid
    PurpleAirAPI requests.
    """

    def __init__(
        self, your_api_read_key=None, your_api_write_key=None, your_ipv4_address=None
    ):
        """
        :param str your_api_read_key: A valid PurpleAirAPI Read key
        :param str your_api_write_key: A valid PurpleAirAPI Write key
        :param str your_ipv4_address: The IPv4 address of your PurpleAir.
        """

        # We can not have all three parameters be empty
        if (
            your_api_read_key is None
            and your_api_write_key is None
            and your_ipv4_address is None
        ):
            raise PurpleAirAPIError(
                "Ensure that the right combination of parameters have been provided! "
                + "`your_api_read_key` or `your_api_write_key` for external internet requests. Or "
                + "just `your_ipv4_address` for local network requests"
            )

        # Save off the API key for internal usage
        self._your_api_read_key = your_api_read_key

        self._base_api_v1_request_string = None

        # Create the base API request string. Must be HTTPS.
        if your_api_read_key or your_api_write_key:
            self._base_api_v1_request_string = "https://api.purpleair.com/v1/"

        # Place holders for information we care about
        self._api_versions = {}
        self._api_keys_last_checked = {}
        self._api_key_types = {}

        retval_api_read_key = None
        retval_api_write_key = None

        if your_api_read_key is not None:
            retval_api_read_key = self._check_an_api_key(your_api_read_key)

        if your_api_write_key is not None:
            retval_api_write_key = self._check_an_api_key(your_api_write_key)

        if your_ipv4_address is not None:
            PurpleAirLocalAPI.__init__(self, ipv4_address_list=your_ipv4_address)

        if retval_api_read_key is not None:
            if self._api_key_types[your_api_read_key] == "READ":
                PurpleAirWriteAPI.__init__(self, api_write_key=your_api_write_key)
                print("PurpleAirAPI: Successfully authenticated read key")

            else:
                raise PurpleAirAPIError("Ensure 'your_api_read_key' is a read key.")

        if retval_api_write_key is not None:
            if self._api_key_types[your_api_write_key] == "WRITE":
                PurpleAirReadAPI.__init__(self, api_read_key=your_api_read_key)
                print("PurpleAirAPI: Successfully authenticated write key")

            else:
                raise PurpleAirAPIError("Ensure 'your_api_write_key' is a write key")

        debug_log(self._api_versions)
        debug_log(self._api_keys_last_checked)
        debug_log(self._api_key_types)
        debug_log(your_ipv4_address)

    def _check_an_api_key(self, str_api_key_to_check):
        """
        An internal class helper method to check if an API key is valid.

        :param str str_api_key_to_check: A valid PurpleAirAPI key to check

        :return True, if an API key can be successfully verified.
        """
        request_url = self._base_api_v1_request_string + "keys"
        the_request_text_as_json = send_url_get_request(
            request_url, api_key_to_use=str_api_key_to_check
        )

        # We good :) get the request information
        self._api_versions[str_api_key_to_check] = the_request_text_as_json[
            "api_version"
        ]
        self._api_keys_last_checked[str_api_key_to_check] = the_request_text_as_json[
            "time_stamp"
        ]
        self._api_key_types[str_api_key_to_check] = the_request_text_as_json[
            "api_key_type"
        ]

        return True

    @property
    def get_api_versions(self):
        """
        A method to return the API versions being used for both read/write keys.
        """

        return self._api_versions

    @property
    def get_api_key_last_checked(self):
        """
        A method to return the timestamp of when the API read/write keys were last checked.
        """

        return self._api_keys_last_checked

    @property
    def get_api_key_type(self):
        """
        A method to return the API key types being used.
        """

        return self._api_key_types



#!/usr/bin/env python3

"""
    Copyright 2024 carlkidcrypto, All rights reserved.
    A python3 file containing helper functions for the PurpleAirAPI** files.
    https://api.purpleair.com/#api-welcome
"""

from purpleair_api.PurpleAirAPIConstants import (
    ACCEPTED_FIELD_NAMES_DICT,
    PRINT_DEBUG_MSGS,
    SUCCESS_CODE_LIST,
    ERROR_CODES_LIST,
)

from purpleair_api.PurpleAirAPIError import PurpleAirAPIError
from requests import get, post, delete
from json import loads


def debug_log(debug_msg_string):
    """
    A helper function to print out
    debug messages only if DEBUG is defined as True in
    'PurpleAirAPIConstants.py'. Messages will be the color
    red.

    :param str debug_msg_string: The debug message string
    """

    if PRINT_DEBUG_MSGS:
        # Make debug messages red using ANSI escape code.
        print("\033[1;31m" + str(debug_msg_string) + "\x1b[0m")


def verify_request_status_codes(status_code) -> bool:
    """
    A helper to check those status codes.
    True if in SUCCESS_CODE_LIST
    False if in ERROR_CODES_LIST
    """

    if status_code in SUCCESS_CODE_LIST:
        return True

    elif status_code in ERROR_CODES_LIST:
        return False

    else:
        raise PurpleAirAPIError(f"Unkown status code - {status_code}!")


def convert_requests_text_to_json(text=None) -> dict:
    """
    A helper to convert request.text to json.

    :param str text: The request.txt to convert to json

    :return dict
    """

    the_request_text_as_json = None
    if text:
        debug_log(f"convert_requests_text_to_json - text: {text}")
        the_request_text_as_json = loads(text)
        debug_log(f"convert_requests_text_to_json - json: {the_request_text_as_json}")

    return the_request_text_as_json


def sanitize_sensor_data_from_paa(paa_return_data) -> dict:
    """
    A helper function.
    Since not all sensors support all field names we check that the keys exist
    in the sensor data. If they do not exist we add it in with a NULL
    equivalent. i.e 0.0, 0, "", etc.
    We access the "sensor" key inside this function.

    :param dict paa_return_data: A dictionary with paa return data
    """

    for key_str in ACCEPTED_FIELD_NAMES_DICT.keys():
        if key_str not in paa_return_data["sensor"].keys():
            paa_return_data["sensor"][key_str] = ACCEPTED_FIELD_NAMES_DICT[key_str]

    return paa_return_data


def send_url_get_request(
    request_url,
    api_key_to_use=None,
    first_optional_parameter_separator=None,
    optional_parameters_dict=None,
):
    """
    A helper to send the url request. It can also add onto the
    'request_url' string if 'optional_parameters_dict' are provided.

    :param str request_url: The constructed string url request string.
    :param str first_optional_parameter_separator: The separator between first parameter
                                                    in optional_parameters_dict. i.e '?' or '&'.
    :param dict optional_parameters_dict: Optional parameters that can be added onto the
                                            request_url.
    """

    if request_url is None:
        raise PurpleAirAPIError(f"A request URL string must be provided")

    if optional_parameters_dict is not None:
        if first_optional_parameter_separator not in ["?", "&"]:
            raise PurpleAirAPIError(
                f"Invalid `first_optional_parameter_separator: {first_optional_parameter_separator}` passed into `send_url_get_request`!"
            )

        opt_param_count = 0
        for opt_param, val in optional_parameters_dict.items():
            if val is not None:
                opt_param_count = opt_param_count + 1

                if opt_param_count == 1:
                    request_url = (
                        request_url
                        + f"{first_optional_parameter_separator}{opt_param}={str(val)}"
                    )

                elif opt_param_count >= 2:
                    request_url = request_url + f"&{opt_param}={str(val)}"

    # Strip any quotes that might persist
    request_url = request_url.replace('"', "")
    # Strip away any whitespace that might persist
    request_url = request_url.replace(" ", "")
    debug_log(request_url)
    my_request = None

    # If any API key is provided use it
    if api_key_to_use is not None:
        my_request = get(request_url, headers={"X-API-Key": str(api_key_to_use)})

    # No API key provided
    else:
        my_request = get(request_url)

    the_request_text_as_json = convert_requests_text_to_json(my_request.text)

    if verify_request_status_codes(my_request.status_code):
        my_request.close()
        del my_request
        return the_request_text_as_json

    else:
        raise PurpleAirAPIError(
            f"""{my_request.status_code}: {the_request_text_as_json['error']} - {the_request_text_as_json['description']}"""
        )


def send_url_post_request(request_url, api_key_to_use, json_post_parameters={}):
    """
    A class helper to send the url request. It can also add onto the
    'request_url' string if 'optional_parameters_dict' are provided.

    :param str request_url: The constructed string url request string.
    """

    debug_log(request_url)
    my_request = None
    if json_post_parameters:
        debug_log(json_post_parameters)
        my_request = post(
            request_url,
            headers={"X-API-Key": str(api_key_to_use)},
            json=json_post_parameters,
        )

    else:
        debug_log(json_post_parameters)
        my_request = post(request_url, headers={"X-API-Key": str(api_key_to_use)})

    the_request_text_as_json = convert_requests_text_to_json(my_request.text)

    if verify_request_status_codes(my_request.status_code):
        my_request.close()
        del my_request
        return the_request_text_as_json

    else:
        raise PurpleAirAPIError(
            f"""{my_request.status_code}: {the_request_text_as_json['error']} - {the_request_text_as_json['description']}"""
        )


def send_url_delete_request(request_url, api_key_to_use, json_post_parameters={}):
    """
    A class helper to send the url request. It can also add onto the
    'request_url' string if 'optional_parameters_dict' are provided.

    :param str request_url: The constructed string url request string.
    """

    debug_log(request_url)
    my_request = None
    if json_post_parameters:
        my_request = delete(
            request_url,
            headers={"X-API-Key": str(api_key_to_use)},
            json=json_post_parameters,
        )

    else:
        my_request = delete(request_url, headers={"X-API-Key": str(api_key_to_use)})

    the_request_text_as_json = convert_requests_text_to_json(my_request.text)

    if verify_request_status_codes(my_request.status_code):
        my_request.close()
        del my_request
        return the_request_text_as_json

    else:
        raise PurpleAirAPIError(
            f"""{my_request.status_code}: {the_request_text_as_json['error']} - {the_request_text_as_json['description']}"""
        )
